# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

import numpy as np
import declarative

from .roots_bin import roots_bin_type
from . import codings
from . import MRF
from . import RDF


coding_mapping_nl = declarative.Bunch(
    #gain_delay    = codings.CodingGainNLDelay,
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingRealnlA,
    num_c         = codings.CodingnlFnlA,
    den_r         = codings.CodingRealnlA,
    den_c         = codings.CodingnlFnlA,
    num_r_u       = None,
    num_c_u       = None,
    den_r_u       = None,
    den_c_u       = None,
    num_collect_r = 1,
    den_collect_r = 1,
    num_collect_c = 1,
    den_collect_c = 1,
)

coding_mapping_nl_safe = declarative.Bunch(
    #gain_delay    = codings.CodingGainNLDelay,
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingRealnlA,
    num_c         = codings.CodingnlFnlA,
    den_r         = codings.CodingRealnlA,
    den_c         = codings.CodingnlFnlA,
    num_r_u       = codings.CodingRealAmp,
    num_c_u       = codings.CodingFA,
    den_r_u       = codings.CodingRealAmp,
    den_c_u       = codings.CodingFA,
    num_collect_r = 1,
    den_collect_r = 1,
    num_collect_c = 1,
    den_collect_c = 1,
)

coding_mapping_FA = declarative.Bunch(
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingRealAmp,
    num_c         = codings.CodingFA,
    den_r         = codings.CodingRealAmp,
    den_c         = codings.CodingFA,
    num_r_u       = None,
    num_c_u       = None,
    den_r_u       = None,
    den_c_u       = None,
    num_collect_r = 1,
    den_collect_r = 1,
    num_collect_c = 1,
    den_collect_c = 1,
)

coding_mapping_RI = declarative.Bunch(
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingRealAmp,
    num_c         = codings.CodingRI,
    den_r         = codings.CodingRealAmp,
    den_c         = codings.CodingRI,
    num_r_u       = None,
    num_c_u       = None,
    den_r_u       = None,
    den_c_u       = None,
    num_collect_r = 1,
    den_collect_r = 1,
    num_collect_c = 1,
    den_collect_c = 1,
)

coding_mapping_SOS = declarative.Bunch(
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingSOS,
    num_c         = codings.CodingSOS,
    den_r         = codings.CodingSOS,
    den_c         = codings.CodingSOS,
    num_r_u       = None,
    num_c_u       = None,
    den_r_u       = None,
    den_c_u       = None,
    num_collect_r = 2,
    den_collect_r = 2,
    num_collect_c = 1,
    den_collect_c = 1,
)

coding_mapping_SOS_safe = declarative.Bunch(
    gain_delay    = codings.CodingGainDelay,
    num_r         = codings.CodingSOSMirror,
    num_c         = codings.CodingSOSMirror,
    den_r         = codings.CodingSOSMirror,
    den_c         = codings.CodingSOSMirror,
    num_r_u       = codings.CodingSOS,
    num_c_u       = codings.CodingSOS,
    den_r_u       = codings.CodingSOS,
    den_c_u       = codings.CodingSOS,
    num_collect_r = 2,
    den_collect_r = 2,
    num_collect_c = 1,
    den_collect_c = 1,
)


def is_unstable(root):
    return abs(root) >= 1


def MRF_generate(
    fitter,
    amplitude_hide  = False,
    frequency_hide  = False,
    delay_hide      = False,
    num_hide        = False,
    den_hide        = False,
    gain_delay_hide = False,
    extra_poles     = None,
    extra_zeros     = None,
    insert_poles    = None,
    insert_zeros    = None,
    coding_mapping  = coding_mapping_nl_safe,
    **kwargs
):
    gain_zeros = []
    if insert_zeros is not None:
        z_r_e, z_c_e, z_c_u, pol = roots_bin_type(
            insert_zeros,
            policy = 'auto',
            F_nyquist = fitter.F_nyquist_Hz
        )
        gain_zeros.extend(z_r_e)
        gain_zeros.extend(z_c_e)
        gain_zeros.extend([r.conjugate() for r in z_c_e])

    gain_poles = []
    if insert_poles is not None:
        p_r_e, p_c_e, p_c_u, pol = roots_bin_type(
            insert_poles,
            policy = 'auto',
            F_nyquist = fitter.F_nyquist_Hz
        )
        gain_poles.extend(p_r_e)
        gain_poles.extend(p_c_e)
        gain_poles.extend([r.conjugate() for r in p_c_e])

    if extra_poles is not None:
        gain_poles.extend(extra_poles)

    if extra_zeros is not None:
        gain_zeros.extend(extra_zeros)

    if gain_poles:
        gain_poles = np.concatenate([fitter.poles, gain_poles])
    else:
        gain_poles = fitter.poles

    gain_delay_coding = coding_mapping.gain_delay()
    gain_delay_coding.setup(
        gain    = fitter.gain,
        delay_s = fitter.delay_s,
    )
    if gain_delay_hide:
        gain_delay_coding.setup(hide_all = True)
    if delay_hide:
        gain_delay_coding.setup(hide_delay = True)

    num_codings = []
    den_codings = []
    z_r, z_c = fitter.zeros_split
    if insert_zeros is not None:
        z_r = z_r + z_r_e
        z_c = z_c + z_c_e
    num_codings = []
    num_codings.extend(
        encode(
            fitter,
            z_r,
            coding_mapping.num_r,
            coding_mapping.num_r_u,
            coding_mapping.num_collect_r
        )
    )
    num_codings.extend(
        encode(
            fitter,
            z_c,
            coding_mapping.num_c,
            coding_mapping.num_c_u,
            coding_mapping.num_collect_c
        )
    )

    p_r, p_c = fitter.poles_split
    if insert_poles is not None:
        p_r = p_r + p_r_e
        p_c = p_c + p_c_e

    den_codings = []
    den_codings.extend(
        encode(
            fitter,
            p_r,
            coding_mapping.den_r,
            coding_mapping.den_r_u,
            coding_mapping.den_collect_r
        )
    )
    den_codings.extend(
        encode(
            fitter,
            p_c,
            coding_mapping.den_c,
            coding_mapping.den_c_u,
            coding_mapping.den_collect_c
        )
    )

    fitterMRF = MRF.MultiReprFilter(
        parent            = fitter,
        num_codings       = num_codings,
        den_codings       = den_codings,
        gain_delay_coding = gain_delay_coding,
        extra_zeros       = extra_zeros,
        extra_poles       = extra_poles,
        **kwargs
    )
    return fitterMRF


def MRF_generate_ZPK(
    data,
    F_Hz,
    SNR,
    ZPK,
    F_nyquist_Hz,
    delay_s        = 0,
    delay_hide     = False,
    coding_mapping = coding_mapping_nl_safe,
    **kwargs
):
    zeros, poles, gain = ZPK
    z_r, z_c, z_u = roots_bin_type(
        zeros,
        policy = 'auto',
        F_nyquist = F_nyquist_Hz,
    )
    if len(z_u) > 0:
        raise RuntimeError("Found unmatched zeros: {0}".format(str(z_u)))
    zeros = []
    zeros.extend(z_r)
    zeros.extend(z_c)
    zeros.extend([r.conjugate() for r in z_c])

    p_r, p_c, p_u = roots_bin_type(
        zeros,
        policy = 'auto',
        F_nyquist = F_nyquist_Hz,
    )
    if len(p_u) > 0:
        raise RuntimeError("Found unmatched poles: {0}".format(str(p_u)))

    poles = []
    poles.extend(p_r)
    poles.extend(p_c)
    poles.extend([r.conjugate() for r in p_c])
    fitter = RDF.RationalDiscFilter(
        F_nyquist_Hz = F_nyquist_Hz,
        poles        = poles,
        zeros        = zeros,
        gain         = gain,
        delay_s      = delay_s,
    )

    return MRF_generate(
        fitter         = fitter,
        coding_mapping = coding_mapping,
        delay_hide     = delay_hide,
    )


def encode(fitter, r_list, st_t, us_t, collect):
    if us_t is None:
        us_t = st_t

    if st_t is not us_t:
        r_list_st = []
        r_list_us = []
        for root in r_list:
            if is_unstable(root):
                r_list_us.append(root)
            else:
                r_list_st.append(root)
    else:
        r_list_st = list(r_list)
        r_list_us = []

    codings = []
    while r_list_st:
        coding = st_t()
        roots = r_list_st[-collect:]
        r_list_st[-collect:] = []
        coding.update_roots(fitter, *roots)
        codings.append(coding)
    while r_list_us:
        coding = us_t()
        roots = r_list_us[-collect:]
        r_list_us[-collect:] = []
        coding.update_roots(fitter, *roots)
        codings.append(coding)
    return codings
