# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

import numpy as np

Ipi  = np.pi * 1j
I2pi = np.pi * 2j


def abs_sq(x):
    return x.real**2 + x.imag**2


class CodingType(object):
    disable = False
    coding_id = None

    def __init__(self):
        #preserved through deepcopy so that codings_ignore set can be maintained
        self.coding_id = id(self)

    def roots(self, sys):
        rc = self.roots_c(sys)
        return self.roots_r(sys) + rc + [r.conjugate() for r in rc]

    def roots_r(self, sys):
        return []

    def roots_c(self, sys):
        return []

    def option_set(self, sys, **kwargs):
        return

    def transfer_abs_sq(self, sys):
        #real/imaginary part of root
        return abs_sq(self.transfer(sys))

    def derivative_wtrans(self, sys):
        return self.transfer(sys), self.derivative(sys)

    def derivative_abs_sq_wtrans(self, sys):
        xfer = self.transfer(sys)
        jac = self.derivative(sys)
        jac_abs_sq = []
        for der in jac:
            jac_abs_sq.append(2 * (der.real * xfer.real + der.imag * xfer.imag))
        return abs_sq(xfer), jac_abs_sq

