# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

import numpy as np
from .base import (
    CodingType,
    #Ipi,
    I2pi
)


class CodingGainDelay(CodingType):
    N_parameters = 2
    p_gain       = 1
    p_delay      = 0
    lock_gain    = False
    hide_gain    = False
    lock_delay   = False
    hide_delay   = False

    def setup(
            self,
            hide_all   = None,
            lock_gain  = None,
            hide_gain  = None,
            lock_delay = None,
            hide_delay = None,
            disable    = None,
            gain       = None,
            delay_s    = None,
    ):
        if hide_all:
            hide_gain = True
            hide_delay = True

        N_parameters = 2
        if lock_gain is not None:
            self.lock_gain = lock_gain
        if hide_gain is not None:
            self.hide_gain = hide_gain
        if self.hide_gain:
            N_parameters -= 1

        if lock_delay is not None:
            self.lock_delay = lock_delay
        if hide_delay is not None:
            self.hide_delay = hide_delay
        if self.hide_delay:
            N_parameters -= 1

        if disable is not None:
            self.disable = disable
        if self.disable:
            N_parameters = 0

        self.N_parameters = N_parameters

        if gain is not None:
            self.p_gain = gain

        if delay_s is not None:
            self.p_delay = delay_s
        return

    def update(self, A = None, B = None):
        if self.disable:
            assert(A is None and B is None)
            return
        if self.hide_delay:
            assert(B is None)
            if self.hide_gain:
                assert(A is None)
            else:
                assert(A is not None)
                gain = A
                if not self.lock_gain:
                    self.p_gain = gain
        else:
            if self.hide_gain:
                assert(A is not None)
                delay = A
                if not self.lock_delay:
                    self.p_delay = delay
            else:
                assert(A is not None)
                assert(B is not None)
                gain, delay = A, B
                if not self.lock_gain:
                    self.p_gain = gain

                if not self.lock_delay:
                    self.p_delay = delay

    def reduce(self):
        if self.disable:
            return []
        if not self.hide_gain:
            if not self.hide_delay:
                return [
                    self.p_gain,
                    self.p_delay
                ]
            else:
                return [self.p_gain]
        else:
            if not self.hide_delay:
                return [self.p_delay]
            else:
                return []

    def transfer(self, sys):
        #gain term (can only be 1 and at the start)
        return self.p_gain * np.exp(-I2pi * self.p_delay * sys.F_Hz)

    def derivative(self, sys):
        if self.disable:
            return []
        if not self.hide_gain:
            if not self.hide_delay:
                return [
                    np.exp(-I2pi * self.p_delay * sys.F_Hz) if not self.lock_gain else 0,
                    self.p_gain * -I2pi * sys.F_Hz * np.exp(-I2pi * self.p_delay * sys.F_Hz) if not self.lock_delay else 0,
                ]
            else:
                return [
                    np.exp(-I2pi * self.p_delay * sys.F_Hz) if not self.lock_gain else 0,
                ]
        else:
            if not self.hide_delay:
                return [
                    self.p_gain * -I2pi * sys.F_Hz * np.exp(-I2pi * self.p_delay * sys.F_Hz) if not self.lock_delay else 0,
                ]
            else:
                return []

    def roots(self, sys):
        return []


