# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

from matplotlib import gridspec
import matplotlib.pyplot as plt

from .. import MRF
from .. import MRF_generate
from ..utilities.mpl import (
    asavefig,
    generate_ax,
)


def plot_fitter_flag(
    self,
    fitter,
    title           = None,
    plot_zp         = True,
    plot_to_nyquist = True,
    with_error      = True,
    with_data_error = True,
    N_points        = 1000,
    gs_base         = gridspec.GridSpec(1, 1)[0],
    fig             = None,
    ax_bunch        = None,
):
    if not isinstance(fitter, MRF.MultiReprFilter):
        fitter = MRF_generate.MRF_generate(fitter)
    axB = generate_ax(ax_bunch)

    if fitter.F_nyquist_Hz > 1.2 * fitter.F_data_max_Hz:
        if fig is None:
            fig = plt.figure()
            fig.set_size_inches(12, 9)
            axB.fig = fig
        gs_DC = gridspec.GridSpecFromSubplotSpec(
            2, 2,
            subplot_spec = gs_base,
            wspace = .2, hspace = .2,
            #height_ratios = height_ratio_list,
            width_ratios = [1, .3],
        )

        gs_data    = gs_DC[0, 0]
        gs_nyquist = gs_DC[1, 0]
        gs_zp      = gs_DC[:, 1]
        zp_horz    = False
    else:
        if fig is None:
            fig = plt.figure()
            fig.set_size_inches(10, 9)
            axB.fig = fig
        gs_DC = gridspec.GridSpecFromSubplotSpec(
            2, 1,
            subplot_spec = gs_base,
            wspace = .1, hspace = .3,
            height_ratios = [1, .7],
        )

        gs_data    = gs_DC[0, 0]
        gs_nyquist = False
        gs_zp      = gs_DC[1, 0]
        zp_horz    = True

    if title is not None:
        axB.fig.suptitle(title)

    if gs_data:
        axB_top = self.plot_fit(
            fitter          = fitter,
            ax_bunch        = None,
            gs_base         = gs_data,
            with_error      = with_error,
            with_data_error = with_data_error,
            plot_to_nyquist = False,
            fig = fig,
        )
        axB.top = axB_top

    if gs_nyquist:
        axB_bottom = self.plot_fit(
            fitter          = fitter,
            ax_bunch        = None,
            gs_base         = gs_nyquist,
            with_error      = False,
            with_data_error = False,
            plot_to_nyquist = True,
            fig             = fig,
            xscale          = 'log',
        )
        axB.bottom = axB_bottom
        left, right = axB.bottom.ax_bottom.get_xlim()
        axB.bottom.ax_bottom.set_xlim(left, fitter.F_nyquist_Hz)

    if gs_zp:
        if fitter.F_nyquist_Hz / max(fitter.F_Hz) > 5:
            axB_right = self.plot_ZP_S(
                fitter          = fitter,
                ax_bunch        = None,
                gs_base         = gs_zp,
                horizontal      = zp_horz,
                fig             = fig,
            )
        else:
            axB_right = self.plot_ZP(
                fitter          = fitter,
                ax_bunch        = None,
                gs_base         = gs_zp,
                horizontal      = zp_horz,
                fig             = fig,
            )
        axB.right = axB_right

    def save(rootname, **kwargs):
        axB << asavefig(rootname, **kwargs)
    axB.save = save

    axB.finalize()
    return axB

def plot_fitter_flag_compare(
    self,
    fitter,
    fitter_ref,
    N_points = 1000,
    gs_base  = gridspec.GridSpec(1, 1)[0],
    fig      = None,
    ax_bunch = None,
):
    if not isinstance(fitter, MRF.MultiReprFilter):
        fitter = MRF_generate.MRF_generate(fitter)
    if not isinstance(fitter_ref, MRF.MultiReprFilter):
        fitter_ref = MRF_generate.MRF_generate(fitter_ref)
    axB = generate_ax(ax_bunch)

    if fig is None:
        fig = plt.figure()
        fig.set_size_inches(14, 9)
        axB.fig = fig
    gs_DC = gridspec.GridSpecFromSubplotSpec(
        3, 3,
        subplot_spec = gs_base,
        wspace = .3, hspace = .2,
        height_ratios = [1, 1, .5],
        width_ratios = [1, .3, .3],
    )

    gs_data    = gs_DC[0, 0]
    gs_nyquist = gs_DC[1, 0]
    gs_zp      = gs_DC[:2, 1]
    gs_zp_ref  = gs_DC[:2, 2]
    zp_horz    = False
    gs_rel     = gs_DC[2, :]

    if gs_data:
        axB_top = self.plot_fit(
            fitter          = fitter,
            gs_base         = gs_data,
            with_error      = True,
            with_data_error = True,
            plot_to_nyquist = False,
            fig             = fig,
            ax_bunch        = None,
        )
        axB.top = axB_top
        axB_top = self.plot_fit(
            fitter          = fitter_ref,
            gs_base         = gs_data,
            with_error      = False,
            with_data_error = False,
            plot_data       = False,
            plot_to_nyquist = False,
            fig             = fig,
            ax_bunch        = axB_top,
            plot_zp         = False,
        )

    if gs_nyquist:
        axB_bottom = self.plot_fit(
            fitter          = fitter,
            ax_bunch        = None,
            gs_base         = gs_nyquist,
            with_error      = False,
            with_data_error = False,
            plot_to_nyquist = True,
            fig             = fig,
            xscale          = 'log',
        )
        axB_bottom = self.plot_fit(
            fitter          = fitter_ref,
            gs_base         = gs_nyquist,
            with_error      = False,
            with_data_error = False,
            plot_data       = False,
            plot_to_nyquist = True,
            fig             = fig,
            xscale          = 'log',
            ax_bunch        = axB_bottom,
            plot_zp         = False,
        )
        axB.bottom = axB_bottom
        left, right = axB.bottom.ax_bottom.get_xlim()
        axB.bottom.ax_bottom.set_xlim(left, fitter.F_nyquist_Hz)

    if gs_zp:
        if fitter.F_nyquist_Hz / max(fitter.F_Hz) > 5:
            axB_right = self.plot_ZP_S(
                fitter          = fitter,
                ax_bunch        = None,
                gs_base         = gs_zp,
                horizontal      = zp_horz,
                fig             = fig,
            )
        else:
            axB_right = self.plot_ZP(
                fitter          = fitter,
                ax_bunch        = None,
                gs_base         = gs_zp,
                horizontal      = zp_horz,
                fig             = fig,
            )
        axB.right = axB_right
        axB.right.ax_poles.set_title('fit roots (s)')

    if gs_zp_ref:
        if fitter.F_nyquist_Hz / max(fitter.F_Hz) > 5:
            axB_right = self.plot_ZP_S(
                fitter          = fitter_ref,
                ax_bunch        = None,
                gs_base         = gs_zp_ref,
                horizontal      = zp_horz,
                fig             = fig,
            )
        else:
            axB_right = self.plot_ZP(
                fitter          = fitter_ref,
                ax_bunch        = None,
                gs_base         = gs_zp_ref,
                horizontal      = zp_horz,
                fig             = fig,
            )
        axB.rright = axB_right
        axB.rright.ax_poles.set_title('reference roots (s)')

    if gs_rel:
        self.plot_rel_comparison(
            fitter          = fitter,
            fitter_ref      = fitter_ref,
            gs_base         = gs_rel,
            fig             = fig,
        )

    def save(rootname, **kwargs):
        axB << asavefig(rootname, **kwargs)
    axB.save = save

    axB.finalize()
    return axB
