# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

import numpy as np
import declarative

def poly_group(F_nyquist):
    if F_nyquist is not None:
        def coeffclean(c):
            c.imag = 0

        def root_transform(roots):
            roots = np.copy(roots)
            select = (abs(roots) > 1)
            #roots[select] = 1 / roots[select].conjugate()
            #just nullify them
            roots = roots[~select]
            return roots

        poly = declarative.Bunch()
        poly.root_transform  = root_transform
        poly.valfromroots    = np.polynomial.polynomial.polyvalfromroots
        poly.roots           = np.polynomial.polynomial.polyroots
        poly.fromroots       = np.polynomial.polynomial.polyfromroots
        poly.val             = np.polynomial.polynomial.polyval
        poly.fit             = np.polynomial.polynomial.polyfit
        poly.vander          = np.polynomial.polynomial.polyvander
        poly.coeffclean      = coeffclean
        return poly
    else:
        #TODO, this isn't working totally yet
        assert(False)

        def root_transform(roots):
            roots = np.copy(roots)
            select = (roots.real > 0)
            roots.real[select] *= -1
            return roots

        def chebvalfromroots(X, roots):
            c = np.polynomial.chebyshev.chebfromroots(roots)
            return np.polynomial.chebyshev.chebval(X, c)

        def coeffclean(c):
            if len(c) >= 2:
                c[::2].imag = 0
                c[1::2].real = 0
            elif len(c) == 1:
                c[::2].imag = 0
            return

        poly = declarative.Bunch()
        poly.root_transform  = root_transform
        poly.valfromroots    = chebvalfromroots
        poly.roots           = np.polynomial.chebyshev.chebroots
        poly.fromroots       = np.polynomial.chebyshev.chebfromroots
        poly.val             = np.polynomial.chebyshev.chebval
        poly.fit             = np.polynomial.chebyshev.chebfit
        poly.vander          = np.polynomial.chebyshev.chebvander
        poly.coeffclean      = coeffclean
        return poly

