"""
"""
from __future__ import division, print_function
import numpy as np

conj_tol = 1e-12
real_tol = 1e-12

def are_conjugates(r1, r2):
    r2 = np.conjugate(r2)
    if abs(r1) < .8:
        return abs(r1 - r2) < conj_tol
    else:
        return abs((r1/r2) - 1) < conj_tol

def is_real(r1):
    return abs(np.imag(r1)) < real_tol


def roots_bin_type(
        roots,
        policy = 'auto',
        F_nyquist = None,
):
    """
    roots_r are the real roots
    roots_c are just the positive complex roots
    roots_u are any unsorted roots
    """
    assert(policy in ['auto', 'pos', 'strict', 'drop_pos', 'drop_neg'])
    if F_nyquist is None:
        key1 = np.real
        key2 = np.imag
    else:
        def key1(val):
            val = val
            if val.imag > 0:
                val = val.conjugate()
            return (np.angle(val), abs(val))
        def key2(val):
            val = val
            if val.imag > 0:
                val = val.conjugate()
            return (abs(val), np.angle(val))

    roots_r = []
    roots_u = []
    seen_cplx_pos = False
    seen_cplx_neg = False

    unsorted_roots = []
    for root in roots:
        if np.imag(root) == 0:
            roots_r.append(np.real(root))
        elif np.imag(root) > 0:
            seen_cplx_pos |= True
            unsorted_roots.append(root)
        else:
            seen_cplx_neg |= True
            unsorted_roots.append(root)

    if policy == 'auto':
        if seen_cplx_neg and seen_cplx_pos:
            policy = 'strict'
        else:
            policy = 'pos'

    roots_c = []
    if policy == 'strict':
        roots = sorted(
            unsorted_roots,
            key = key1,
        )
        unsorted_roots = []
        idx = 0
        while idx < len(roots):
            root = roots[idx]
            if (len(roots) - idx > 1) and are_conjugates(root, roots[idx + 1]):
                if np.imag(root) > 0:
                    roots_c.append(root)
                else:
                    roots_c.append(roots[idx + 1])
                idx += 2
            else:
                unsorted_roots.append(root)
                idx += 1

        roots = sorted(
            unsorted_roots,
            key = key2,
        )
        idx = 0
        while idx < len(roots):
            root = roots[idx]
            root2 = roots[-idx-1]
            if (len(roots) - idx > 1) and are_conjugates(root, root2):
                if np.imag(root) > 0:
                    roots_c.append(root)
                else:
                    roots_c.append(root2)
                idx += 2
            else:
                if is_real(root):
                    roots_r.append(np.real(root))
                    idx += 1
                else:
                    roots_u.append(root)
                    idx += 1
    elif policy == 'pos':
        for root in unsorted_roots:
            if np.imag(root) > 0:
                roots_c.append(root)
            else:
                roots_c.append(root.conjugate())
    elif policy == 'drop_neg':
        for root in unsorted_roots:
            if np.imag(root) > 0:
                roots_c.append(root)
    elif policy == 'drop_pos':
        for root in unsorted_roots:
            if np.imag(root) < 0:
                roots_c.append(root.conjugate())
    return roots_r, roots_c, roots_u, policy
