"""
Contains setup functions for test data, these are returned in a QuickBunch dictionary, with some annotation about the number of data sets
"""
from __future__ import division, print_function, unicode_literals
import numpy as np

from ..utilities.np import logspaced
from . import utilities


def meta_generator(Z, P, K):
    def dataset_gen(
            sN  = 0,
            iN  = 0,
            SNR = 10,
            invert = False,
            **kwargs
    ):
        if not invert:
            ZPK_s = (Z, P, K)
        else:
            ZPK_s = (P, Z, 1/K)

        N = 1000
        if sN == 0:
            F_Hz = np.linspace(0, 10, N)
            F_nyquist_Hz = 1024
        elif sN == 1:
            F_Hz = logspaced(.01, 10, N)
            F_nyquist_Hz = 1024
        elif sN == 2:
            F_Hz = np.linspace(0, 100, N)
            F_nyquist_Hz = 10000
        else:
            F_Hz = logspaced(.01, 100, N)
            F_nyquist_Hz = 10000

        return utilities.generator_autofill(
            F_Hz          = F_Hz,
            SNR           = SNR,
            F_nyquist_Hz  = F_nyquist_Hz,
            bestfit_ZPK_s = ZPK_s,
            sN            = sN,
            iN            = iN,
        )
    return dataset_gen


desc_simple1 = utilities.make_description(
    generator = meta_generator(
        (-1, -1, -1, -1, -1),
        (-.1, -.1, -.1),
        1
    ),
    sets      = 4,
    instances = -1,
)


desc_simple2 = utilities.make_description(
    generator = meta_generator(
        (-1, -1, -.1 + 1.1j, -.1 - 1.1j, -1),
        (-.1, -.1, -.1 + 3j, -.1 - 3j, -.01 + 1j, -.01 - 1j, -1,),
        1
    ),
    sets      = 4,
    instances = -1,
)


desc_simple3 = utilities.make_description(
    generator = meta_generator(
        (-2, -2,),
        (-.1,  -.1, -.1,),
        1
    ),
    sets      = 4,
    instances = -1,
)


desc_simple4 = utilities.make_description(
    generator = meta_generator(
        (-.1,),
        (-1, -1,),
        1
    ),
    sets      = 4,
    instances = -1,
)
