# -*- coding: utf-8 -*-
"""
"""
from __future__ import division, print_function, unicode_literals

import numpy as np
import declarative

from ..fake_aid import ensure_aid


def nyquist_move(
    fitter,
    nyquist_new,
    clear_neg_real = False,
    split_neg_real = False,
    method         = 'exp',
    aid            = None,
):
    """
    Modifies a fitter IN PLACE to use a different nyquist frequency
    """
    aid = ensure_aid(aid)

    nyquist_old = fitter.F_nyquist_Hz
    Z_list = fitter.zeros
    P_list = fitter.poles
    op = declarative.Bunch(
        gain_flip = 1
    )

    #currently a restriction in the algorithm, it can't handle aliasing of poles (need bilinear for that)
    assert(nyquist_new > nyquist_old)
    ratio = nyquist_new / nyquist_old

    if method == 'exp':
        def root_list_map(root_list, rtype):
            new_list = []
            for root in root_list:
                new_roots = None
                if abs(root.imag) < 1e-15:
                    #aid.log(root)
                    if root.real > 0:
                        if root.real < 1:
                            new_roots = [1 - (1 - root.real) / ratio]
                        else:
                            #actually, just clear it
                            aid.log("Cleared ", rtype, root)
                            #new_roots = []
                            #flip it then insert
                            op.gain_flip *= -1
                            root = 1/root
                            #new_roots = [1 - (1 - root.real) / ratio]
                            new_roots = []
                    else:
                        if clear_neg_real:
                            aid.log("Cleared ", rtype, root)
                            new_roots = []
                        elif split_neg_real:
                            angle = np.pi / ratio
                            amp = -root.real
                            if amp < 1:
                                amp = 1 - (1 - amp) / ratio
                            else:
                                amp = (amp - 1) / ratio + 1
                            new_roots = [
                                amp * np.exp(1j * angle),
                                amp * np.exp(-1j * angle)
                            ]
                        else:
                            new_roots = [-1 - (-1 - root.real) / ratio]
                else:
                    amp = abs(root)
                    angle = np.angle(root)
                    if angle > 0:
                        angle = angle / ratio
                    else:
                        angle = angle / ratio
                    if amp < 1:
                        amp = 1 - (1 - amp) / ratio
                    else:
                        amp = (amp - 1) / ratio + 1
                    new_roots = [amp * np.exp(1j * angle)]
                new_list.extend(new_roots)
            return new_list
    elif method == 'bilinear':
        def root_list_map(root_list, rtype):
            new_list = []
            for root in root_list:
                new_root = (ratio * (1 + root) + (root - 1))/(ratio * (1 + root) - (root - 1))
                new_list.append(new_root)
            return new_list

    fitter.F_nyquist_Hz = nyquist_new
    #these are OK since the nyquist was low
    #h_ref_pre = abs(fitter.poly.valfromroots(fitter.Xp_ref, Z_list))
    #h_ref_pre /= abs(fitter.poly.valfromroots(fitter.Xp_ref, P_list))
    #aid.log("GAIN: ", fitter.gain)
    poles = root_list_map(P_list, rtype = 'pole')
    zeros = root_list_map(Z_list, rtype = 'zero')

    h_ref_post = 1
    group_len = 1
    if len(poles) > len(zeros):
        zeros = list(zeros) + [0] * (len(zeros) - len(poles))
    else:
        poles = list(poles) + [0] * (len(poles) - len(zeros))
    zeros = np.asarray(zeros)
    poles = np.asarray(poles)
    mlen = max(len(poles), len(zeros))
    for idx in range((mlen - 1) // group_len + 1):
        zs = zeros[idx * group_len : (idx + 1) * group_len]
        ps = poles[idx * group_len : (idx + 1) * group_len]
        h_ref_post *= abs(fitter.poly.valfromroots(fitter.Xp_ref, zs))
        h_ref_post /= abs(fitter.poly.valfromroots(fitter.Xp_ref, ps))

    #this is UGLY but necessary for the numerical stability
    #use set_raw without a setter so that it doesn't try to update the gain and become unstable
    fitter.set_raw('poles', poles, setter = lambda s, x : x)
    fitter.set_raw('zeros', zeros, setter = lambda s, x : x)
    #fitter.gain = fitter.gain * h_ref_post / h_ref_pre * op.gain_flip
    #aid.log("GAIN FLIP: ", op.gain_flip)
    h_ref_pre = fitter.h_b_ref / fitter.h_a_ref
    fitter.gain = fitter.gain / abs(h_ref_post / h_ref_pre) * op.gain_flip
    #aid.log("GAIN: ", fitter.gain)
    return

